package protocolsupport.protocol.utils.i18n;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.stream.Collectors;

import protocolsupport.utils.Utils;

public class I18NData {

	public static final String DEFAULT_LOCALE = "en_us";

	private static final HashMap<String, I18N> i18ns = new HashMap<>();

	static {
		//the languages file is generated by the gradle build script
		new BufferedReader(new InputStreamReader(Utils.getResource("i18n/languages"))).lines().forEach(I18NData::loadAndRegisterI18N);
	}

	private static void loadAndRegisterI18N(String locale) {
		loadAndRegisterI18N(locale, Utils.getResource("i18n/" + locale + ".lang"));
	}

	public static void loadAndRegisterI18N(String locale, InputStream stream) {
		i18ns.put(locale, loadI18N(locale, stream));
	}

	public static I18N loadI18N(String locale, InputStream stream) {
		return new I18N(locale, new BufferedReader(new InputStreamReader(stream)).lines().collect(Collectors.toList()));
	}

	public static I18N getI18N(String locale) {
		return i18ns.getOrDefault(locale, i18ns.get(DEFAULT_LOCALE));
	}

	public static String i18n(String locale, String key, String... args) {
		return String.format(getI18N(locale).getI18N(key), (Object[]) args);
	}

}
